import bpy, os
import shutil
from pathlib import Path
from . import bl_info

class InstallMatcaps(bpy.types.Operator):
    """Install Matcaps from addon folder"""
    bl_idname = "shading_plus.install_matcaps"
    bl_label = "Install Matcaps"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        addon_dir = Path(__file__).parent
        matcap_dir = os.path.join(addon_dir, "matcaps")
        matcap_dir = Path(matcap_dir)  # Convert back to Path object for consistency

        if not matcap_dir.exists():
            self.report({'ERROR'}, f"Matcap directory not found: {matcap_dir}")
            return {'CANCELLED'}

        # Allowed file extensions
        allowed_extensions = ['.png', '.jpg', '.jpeg', '.exr']

        matcap_files = [f for f in matcap_dir.iterdir() if f.is_file() and f.suffix.lower() in allowed_extensions]

        if not matcap_files:
            self.report({'WARNING'}, "No matcap files found in the addon's matcaps folder.")
            return {'CANCELLED'}

        installed_matcaps = []
        failed_matcaps = []

        # Define Blender's studiolights matcap directory using bpy.utils.user_resource
        user_path = Path(bpy.utils.resource_path('USER'))
        user_studiolights = Path(os.path.join(user_path, "datafiles", "studiolights", "matcap"))
        user_studiolights.mkdir(parents=True, exist_ok=True)

        for matcap_file in matcap_files:
            matcap_name = matcap_file.name
            destination = user_studiolights / matcap_name

            try:
                # Remove existing matcap file if it exists
                if destination.exists():
                    destination.unlink()
                    print(f"Removed existing matcap: {destination}")

                # Also remove any existing studio light entries with the same name
                for studio_light in bpy.context.preferences.studio_lights:
                    if studio_light.name == matcap_name and studio_light.type == 'MATCAP':
                        bpy.context.preferences.studio_lights.remove(studio_light)
                        print(f"Removed existing studio light entry: {matcap_name}")

                # Copy matcap file
                shutil.copy2(matcap_file, destination)
                print(f"Copied matcap: {matcap_name} to {destination}")

                installed_matcaps.append(matcap_name)
            except Exception as e:
                print(f"Failed to copy or add matcap {matcap_name}: {e}")
                failed_matcaps.append(matcap_name)

        # Force Blender to refresh its list of studio lights by loading from the directory
        for matcap_file in user_studiolights.iterdir():
            if matcap_file.suffix.lower() in allowed_extensions:
                try:
                    new_light = bpy.context.preferences.studio_lights.load(str(matcap_file), 'MATCAP')
                    studio_light_name = new_light.name  # Use full name with extension
                    print(f"Loaded Matcap: {studio_light_name}")
                except Exception as e:
                    print(f"Failed to load Matcap '{matcap_file.name}': {e}")

        # Feedback to the user
        if installed_matcaps:
            self.report({'INFO'}, f"Successfully installed matcaps: {', '.join(installed_matcaps)}")
        if failed_matcaps:
            self.report({'WARNING'}, f"Failed to install matcaps: {', '.join(failed_matcaps)}")

        return {'FINISHED'}




def register():
    bpy.utils.register_class(InstallMatcaps)
def unregister():
    bpy.utils.unregister_class(InstallMatcaps)

if __name__ == "__main__":
    register()
