import bpy
from mathutils import Color

# Define color categories with adjusted "black" value
COLOR_CATEGORIES = {
    "white": Color((1.0, 1.0, 1.0)),
    "light_grey": Color((0.7, 0.7, 0.7)),
    "dark_grey": Color((0.3, 0.3, 0.3)),
    "black": Color((0.02, 0.02, 0.02)),  # Adjusted black to a very dark grey
    "red": Color((1.0, 0.0, 0.0)),
    "blue": Color((0.0, 0.0, 1.0)),
    "purple": Color((0.5, 0.0, 0.5)),
    "magenta": Color((1.0, 0.0, 1.0)),
    "yellow": Color((1.0, 1.0, 0.0)),
    "green": Color((0.0, 1.0, 0.0)),
    "orange": Color((1.0, 0.5, 0.0)),
    "cyan": Color((0.0, 1.0, 1.0)),
}

def classify_color(color):
    """Classify color into categories based on refined HSV values."""
    h, s, v = color.hsv

    # Refined classification based on brightness (v)
    if 0.0 <= v <= 0.05:
        return "black"
    elif 0.05 < v <= 0.1:
        return "dark_grey"
    elif 0.1 < v <= 0.5:
        return "light_grey"
    elif 0.5 < v <= 1.0:
        return "white"
    elif h < 0.1 or h >= 0.9:
        return "red"
    elif 0.1 <= h < 0.2:
        return "orange"
    elif 0.2 <= h < 0.4:
        return "yellow"
    elif 0.4 <= h < 0.6:
        return "green"
    elif 0.6 <= h < 0.7:
        return "cyan"
    elif 0.7 <= h < 0.8:
        return "blue"
    elif 0.8 <= h < 0.9:
        return "magenta"
    else:
        return "grey"  # Default to grey for edge cases

def get_material_base_color(material):
    """Get the base color of the material from its nodes, including within group nodes."""
    def find_color_in_group(node_group):
        # Recursively search within a node group for a color input
        for node in node_group.nodes:
            if node.type == 'BSDF_PRINCIPLED':
                return node.inputs['Base Color'].default_value[:3]
            elif node.type == 'GROUP' and "Color" in node.inputs:
                return node.inputs["Color"].default_value[:3]
        return None

    if material.use_nodes:
        # Look for the BSDF or main color node
        for node in material.node_tree.nodes:
            if node.type == 'BSDF_PRINCIPLED':
                return node.inputs['Base Color'].default_value[:3]
            elif node.type == 'GROUP':
                # If it's a group, search within the group
                color = find_color_in_group(node.node_tree)
                if color:
                    return color
    # If no node-based color, use default base color
    return material.diffuse_color[:3]

class ApplyViewportColors(bpy.types.Operator):
    """Simplify all material viewport display colors to a limited color range, perfect for achieving a cell-shading effect with the Cell Solid Preset."""
    bl_idname = "shading_plus.apply_viewport_colors"
    bl_label = "This will override your materials viewport display colors. Are you sure?"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        for material in bpy.data.materials:
            # Get the base color of the material
            base_color = Color(get_material_base_color(material))

            # Classify the color and get the representative color
            category = classify_color(base_color)
            viewport_color = COLOR_CATEGORIES.get(category, Color((1, 1, 1)))

            # Apply to viewport display color
            material.diffuse_color = (*viewport_color[:3], 1.0)

        self.report({'INFO'}, "Viewport colors applied based on categories.")
        return {'FINISHED'}

    def invoke(self, context, event):
        return context.window_manager.invoke_confirm(self, event)

# Registering the operator
def register():
    bpy.utils.register_class(ApplyViewportColors)

def unregister():
    bpy.utils.unregister_class(ApplyViewportColors)

if __name__ == "__main__":
    register()
